﻿using Microsoft.EntityFrameworkCore;
using ProRestaurant.Classes;
using ProRestaurant.Containers;
using ProRestaurant.DBContexts;
using ProRestaurant.Models.Restaurants;
using System.Collections.Generic;
using System.Linq;

namespace ProRestaurant.Repository.Restaurants
{
    public class RestaurantCategoryRepository : IRestaurantCategoryRepository
    {
        private readonly DBContext dBContext;

        public RestaurantCategoryRepository(DBContext _DB)
        {
            dBContext = _DB;
        }

        public ICollection<RestaurantCategory> Get()
        {
            var cats = dBContext.RestaurantCategories.OrderBy(c => c.Description).ToList();

            foreach (var cat in cats)
            {
                if (!string.IsNullOrEmpty(cat.Image) && !cat.Image.Contains("data:image"))
                    cat.Image = ImageFormatter.ImageToBase64(cat.Image);
            }

            return cats;
        }

        public List<Category> GetCategories()
        {
            List<Category> categories = new List<Category>();
            var cats = dBContext.RestaurantCategories.OrderBy(c => c.Description).ToList();
            foreach (var cat in cats)
                categories.Add(new Category() { Description = cat.Description });

            return categories;
        }

        public void Insert(RestaurantCategory RestaurantCategory)
        {
            dBContext.Add(RestaurantCategory);
            Save();
        }

        public void Remove(int id)
        {
            var category = dBContext.RestaurantCategories.Where(r => r.Id == id).FirstOrDefault();
            dBContext.Remove(category);
            Save();
        }

        public void Save()
        {
            dBContext.SaveChanges();
        }

        public void Update(RestaurantCategory RestaurantCategory)
        {
            dBContext.Entry(RestaurantCategory).State = EntityState.Modified;
            Save();
        }
    }
}
